.jetpack-display-remote-posts {
	margin: 5px 0 20px 0;
}

.jetpack-display-remote-posts h4 {
	margin: 5px 0;
	padding: 0;
}

.jetpack-display-remote-posts p {
	margin: 0;
	padding: 0;
}

.jetpack-display-remote-posts img {
	max-width: 100%;
}
