.jetpack-social-icons-widget-item {
	background: #fff;
	border: 1px solid #dcdcde;
	cursor: move;
	margin: 0;
}

html[class*='wordpress_com'] .jetpack-social-icons-widget-item,
.in-calypso .jetpack-social-icons-widget-item {
	border-color: #c8d7e1;
}

.jetpack-social-icons-widget-item:hover {
	outline: 1px solid #999;
	outline-offset: -1px;
}

html[class*='wordpress_com'] .jetpack-social-icons-widget-item:hover,
.in-calypso .jetpack-social-icons-widget-item:hover {
	outline-color: #a8bece;
}

.jetpack-social-icons-widget-item.ui-sortable-helper {
	border-color: #999;
}

html[class*='wordpress_com'] .jetpack-social-icons-widget-item.ui-sortable-helper,
.in-calypso .jetpack-social-icons-widget-item.ui-sortable-helper {
	border-color: #a8bece;
}

.jetpack-social-icons-widget-item.ui-sortable-helper {
	box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
}

.jetpack-social-icons-widget-item.ui-state-placeholder {
	border-color: #a7aaad;
	border-style: dashed;
	margin: 0 0 1px;
}

.jetpack-social-icons-widget-item + .jetpack-social-icons-widget-item:not(.ui-state-placeholder) {
	margin-top: -1px;
}

.jetpack-social-icons-widget-item-wrapper {
	padding: 1em;
	position: relative;
}

.jetpack-social-icons-widget-item .handle {
	display: block;
	height: 100%;
	left: 0;
	position: absolute;
	top: 0;
	width: 100%;
}

.jetpack-social-icons-widget-item p {
	margin: 0;
	position: relative;
}

.jetpack-social-icons-widget-item p + p {
	margin-top: 1em;
}

.jetpack-widget-social-icons-remove-item {
	display: inline-block;
}

.jetpack-widget-social-icons-remove-item-button {
	color: #d63638;
	text-decoration: none;
}

.jetpack-widget-social-icons-remove-item-button:focus,
.jetpack-widget-social-icons-remove-item-button:hover {
	color: #f00;
}

.jetpack-social-icons-add-button:before {
	content: "\f132";
	display: inline-block;
	position: relative;
	left: -2px;
	top: -1px;
	font: 400 20px/1 dashicons;
	vertical-align: middle;
	transition: all 0.2s;
	-webkit-font-smoothing: antialiased;
	-moz-osx-font-smoothing: grayscale;
}
