.contact-map {
	max-width: 100%;
	border: 0;
}

.contact-map-api-error {
	border-left-color: #dba617;
	border-left-style: solid;
	border-left-width: 4px;
	box-shadow: 0 1px 1px 0 rgba(0,0,0,.1);
	margin: 5px 0 15px;
	padding: 1px 12px;
}
